module Iowa

class Context

	attr_accessor :request, :response, :do_not_render, :phase
	attr_accessor :actionID, :requestID, :sessionID
	attr_accessor :session, :radio_box_id, :prior_exception
	
	MatchURL = Regexp.new('(/*(?:[^/]*/*)*?)(\w+\-\w+\-\w+\.\w+\.[\w\.]+)*(-\d+)?$')

	def initialize(request, out)
		@request = request
		url = @request.uri
		match = MatchURL.match url
		@urlRoot = match[1]
		remainder = match[2]? match[2] : ""
		@urlRoot.sub!(/\/$/,C_empty)
		@sessionID, @requestID, @actionID = remainder.split(C_dot,3)
		@headers = request.headers_in
		@response = out
		@radio_box_id = {}
		@do_not_render = false
		self.element_attributes = {}
		reset!
	end

	def element_attributes
		@element_attributes
	end
	
	def element_attributes=(attribute)
		@element_attributes = attribute
	end
	
	def baseURL
		"#{@urlRoot}"
	end
	
	def sessionURL
		Iowa.app.location.to_s != C_empty ?
			"#{@urlRoot}/#{@sessionID}-#{Iowa.app.location}" :
			"#{@urlRoot}/#{@sessionID}"
	end

	def actionURL
		Iowa.app.location.to_s != C_empty ?
			"#{@urlRoot}/#{@sessionID}.#{@requestID}.#{elementID}-#{Iowa.app.location}" :
			"#{@urlRoot}/#{@sessionID}.#{@requestID}.#{elementID}"
	end

	def reset!
		@elementID = []
		pushElement
		
		@changes = []
        @actionContext = []
        
		@rootStack = []
		
		@action = nil
		
		@state = {}
		@lastBinding = nil
		@lastRoot = nil
	end
	
	def pushElement
		@elementID.push "1"
	end
	
	def popElement
		@elementID.pop
	end
	
	def nextElement!
		@elementID.last.next!
	end
	
	def elementID
		@elementID.join(C_dot)
	end
	
	def elementID=(val)
		@elementID[-1] = val
	end
	
	def setBindingNow(binding, value)
		binding.set(@rootStack.last, value)
		setBinding(binding, value)
	end
    
	def setBinding(binding, value)
		if(@lastBinding == binding && @lastRoot == root)
			@changes.pop
		end
		@changes.push [root, binding, value]
		@lastBinding = binding
		@lastRoot = root
	end
	
	def getBinding(binding)
		cache = @phase == :handleRequest ? true : false
		binding.get(root,cache) if binding
	end

	def testBinding(binding)
		binding.test(root)
	end

	def createAccessor(binding)
		unless root.class.method_defined?(binding.association.to_sym)
			root.class.__send__(:attr_accessor, binding.association.to_sym)
		end
		rescue Exception
	end
	
	def commit(changes)
		changes.each do |target, binding, value|
			binding.set(target, value)
		end
	end
	
	def setAction(binding)
		@action = [root, binding.get(root)]
		@actionContext = @changes.dup
	end
	
	def invokeAction
		raise IgnoreRequest unless @action
		
		nextPage = nil
		page, method = @action
    	commit @changes
        commit @actionContext
		page.invokeAction(method) do |newPage|
			nextPage = newPage
		end
		reset!
		
		nextPage
	end
	
	def pushRoot(root)
		@rootStack.push root
	end
	
	def popRoot
		@rootStack.pop
	end
	
	def root
		@rootStack.last
	end
	
	def [](key)
		@state[key]
	end
	
	def []=(key, val)
		@state[key] = val
	end
end

end
