require 'rubygems' 
require 'utils'
require 'logger'
require 'json/objects'
require 'json/lexer'
require 'yaml'
require 'needle'
require 'webrick'


include WEBrick



module Orbjson

  
  $roy_logger = Logger.new( "Orbjson.log" )
  $roy_logger.level = Logger::DEBUG

  Version = "0.0.4"
  STDERR.puts( "Orbjson: version #{Version }")
  LibPath = File.dirname(__FILE__)  
   
  
  # The System class holds the core broker methods. It is reponsible for 
  # registering server-side classes, sending a list of available services
  # to the client, and connecting JSON-RPC requests with registered services
  #
  # The System object is initialized by calling +init+ and passing in a
  # mapping of files and classes implementing services.
  class System
    @@registry = Needle::Registry.new() 
   
    # Returns an object stored in the registry
    def self.get_object( klass, *args )
      return @@registry[ klass.intern ] 
    end
    
    # Adds an object to the registry
    def self.put_object( klass, instance )
      @@registry.register( klass ) { instance }
    end
    
    # Returns a list of registered service classes. 
    def self.registered_classses
      @@registry.keys.map{ |k| 
       $roy_logger.debug( "[#{__LINE__}] self.registered_classses  has @@registry[k].to_s   #{@@registry[k].to_s}" ) 
        cls = @@registry[k].class.to_s 
        ( cls  =~ /Needle::/ || cls  =~ /^Hash$/ || cls  =~ /^[a-z]/ )  ?  nil : @@registry[k].class.to_s
      }.compact

    end
   
    # Returns a list of registerd services.  Wraps a call to System.list_methods.
    # The format of each listed service name is className.method
    # It was added as a conveience for client code that uses the archaic
    # camelCase method naming convention found in 2oth C. programming languages
    def listMethods
      System.list_methods
    end
    
      # Returns a list of registerd services.  
      # The format of each listed service name is +className.method+    
    def self.list_methods
      mlist = []
      
      System.registered_classses.each  do |cls|
        name = String.new( cls )
        name[0]  = name[0].chr.downcase
        begin 
          $roy_logger.debug( "[#{__LINE__}] System#list_methods has cls   #{cls}" )
          klass = eval( cls )
          ( klass.public_instance_methods  - 
            Object.methods ).sort.each { |m| 
            $roy_logger.debug( "[#{__LINE__}] System#list_methods adding  #{name}.#{m}" )
              mlist << "#{name}.#{m}" 
           }  
         rescue Exception
           STDERR.puts( "[#{__LINE__}] System#list_methods error #{$!}" )
           $roy_logger.error( "[#{__LINE__}] System#list_methods error #{$!}" )
         end
      end
     $roy_logger.debug( "[#{__LINE__}] System#list_methods returning #{mlist.inspect}" )      
      mlist 
    end

      # You can configure the Orbjson services list in a few ways, passing
      # detials into the init method.  
      #
      # Well, three, really:
      #  1. Pass in a path to a local YAML file; the path must begin with
      #   'file://'
      #     For example: cfg = 'file://config.yaml'
      #     System.init( cfg )
      #     
      #  2. Pass in some YAML:   
      #    cfg = 'services/sample: 
      #            - Details'
      #    System.init( cfg )
      #            
      #  3. Pass in some an actual Hash object:   
      #    cfg = { 'services/sample' => ['Details'] }
      #    System.init( cfg )
      #
      #  The hash (however you express it) consists of 'require' paths mapped
      #  to arrays of classes to instantiate.
      #
    def self.init( cfg, flush = true )
      
      @@registry = Needle::Registry.new() if flush 
      STDERR.puts( "cfg starts as  #{cfg}") 
      case  cfg.class.to_s
        when 'String'
          if cfg =~ /^file:\/\//
            cfg.gsub!( /^file:\/\//, '' )
            STDERR.puts( "cfg is now #{cfg}")
            config_hash = YAML::load( File.open( cfg  ) )
          else
            STDERR.puts( "cfg is  #{cfg}")
            config_hash = YAML::load( cfg  )            
          end
        when 'Hash'
          config_hash = cfg
        else
          raise "init was given unusable configuration data [ #{cfg.class}]: #{$!}"
      end
      
      config_hash.each do |src_file, class_list|
        require src_file
        class_list.each do |klass|
          @@registry.register( klass ) { new_from_name( klass ) }
        end
      end  

      @@registry.register( :System ) { System.new }
      @@registry.register( :system ) { System.new }  
      $roy_logger.debug(  "@@registry now has \n" + @@registry.keys.inspect )
    end
  end

    # Assorted method used by both CGI and WEBrick Orbjson classes
  module Common

      # Takes the result of some method, plus the id of the originating
      # JSON-RP call, and creates the JSON-RPC reponse
    def format_repsonse( result, error, id )
      id.class == String ? quote = '"' : quote = ''
      '{ "result" : ' + result.to_json + ', "error":"' + 
       error.to_s + '", "id": ' +  quote +  id.to_s + 
       quote + ' }'
    end

      # Takes a JSON-RPC message, figures out what it means in Ruby,
      # invokes the request method on the correspnding service, and returns
      # the results
    def process( json )
      json   = "#{json}"
      null=nil;  
      jsonobj = {}.from_json( JSON::Lexer.new(json ) ) 
      klass, method = jsonobj[ 'method'].split( '.' ) 
      $roy_logger.debug( "klass, method  = #{klass}, #{method}" )
      receiver = System.get_object( klass.capitalize )
      begin
        if ( jsonobj[ 'params'] && jsonobj[ 'params'] != 'nil' )
          if jsonobj[ 'params'].size < 1
            result = receiver.send( method  )                  
          else
            result = receiver.send( method , *jsonobj[ 'params']   )
          end
        else
          result = receiver.send( method  )        
        end
        error = null
      rescue
        result = null
        error = $!.to_s
      end
      format_repsonse( result, error, jsonobj['id'] || 'default' )
    end
  end


    # Manages json-rpc requests sent as a post to a WEBrick servlet.
    # Mount a path to  WEBrick_JSON_RPC, and you should be set
  class WEBrick_JSON_RPC < HTTPServlet::AbstractServlet
    
    include  Common
   
      # This initiats the actual message processing.  Calls into the Common
      # menthod 'process'
    def process_request( req, res, json )
      res.body = process( json )
      res['Content-Type'] = "text/javascript"    
    end
   
      # Delegates the call to process_request
    def do_GET( request, response )
      process_request( request, response, request.query_string )
    end
    
      # Delegates the call to process_request
    def do_POST( request, response )
      process_request( request, response, request.body )    
    end  
     
  
  end


    # Manages json-rpc requests sent as a raw post to a CGI application.
  class CGI_JSON_RPC 
    
    include  Common

      # Passes the configuration off to System.init
    def initialize(  cfg  )
       System.init( cfg  )
    end
   
      # Takes the raw post data, grabs the JSON-RPC message, and starts
      # the processing
    def process_request( )
      cgi =  CGI.new
      json   = cgi.raw_post
      process( json )
    end
  
  end
end

#
#
#  TODO  See specs at http://www.json-rpc.org/specs.xhtml
#  1.3 notification
# A notification is a special request which does not have a response.
# The notification is a single object serialized using JSON.
# It has the same properties as the request object with one exception.
# 
#     * id
#       Muste be null.
#
# 2.2   JSON-RPC over HTTP
# With some limitations HTTP can be used for communicating to a service.
# A client may send one or more requests to a service.
# The requests must be sent as an HTTP POST containing all serialized request objects.
# The reply must contain the serialized response objects for all requests.
# Notifications may be sent by the client or the service.
# 
# A non valid request must result in closing the connection.
# A non valid responses must raise an exception for all unanswered requests on the client.
# Closing a connection must raise an exception for all unanswered requests on the client.
#
# 3. JSON Class hinting?
# There are only simple data types defined in JSON.
# To overcome this problem in a JSON compatible way a special property for objects is introduced.
# {"jsonclass":["constructor", [param1, ], "prop1": ...}
# The object is then instanciated using the constructor passing the parameters.
# Once constructed the properties will be applied.


# Copyright (C) 2005 James Britt & Neurogami LLC
# Big thanks to Florian Franks for his ruby-json lib, and Jamis Buck for
# Needle
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA.

